from __future__ import unicode_literals

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from evoteusers.forms import EvoteAuthenticationForm

from eVotUM.Cripto import cautils

from .models import Page, SecretEntry
from .templatetags.admin_tags import is_private_key
from evotecore.widgets import TinymceTextarea


class AdminAuthenticationForm(EvoteAuthenticationForm):
    """Admin user must have is_staff=True"""

    def confirm_login_allowed(self, user, cleaned_data=None):
        super(AdminAuthenticationForm, self).confirm_login_allowed(user)
        if not user.is_staff:
            raise forms.ValidationError(
                self.error_messages['invalid_login'],
                code='invalid_login',
            )


class PageForm(forms.ModelForm):
    """ """

    class Meta:
        model = Page
        fields = ['title_pt', 'title_en', 'body_pt', 'body_en']
        widgets = {
            'body_pt': TinymceTextarea,
            'body_en': TinymceTextarea
        }


class SecretEntryForm(forms.ModelForm):
    """
    The form for a single certificate entry
    """
    document = forms.FileField(required=False)
    password = forms.CharField(
        label="password",
        widget=forms.PasswordInput(
            attrs={
                'placeholder': _(u'Insert the private key password')
            }
        ),
        required=False
    )

    class Meta:
        model = SecretEntry
        fields = ('identifier', 'body', 'password')

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request', None)
        super(SecretEntryForm, self).__init__(*args, **kwargs)
        # Making identifier disabled
        self.fields['identifier'].disabled = True

    def clean_body(self):

        body = self.cleaned_data.get('body', None)

        # Different behaviour depending on being on a formset or a simple form
        if self.prefix:
            document_key = self.prefix + '-document'
        else:
            document_key = 'document'

        document = self.files.get(document_key, None)

        if document is not None:
            body = document.read()

        return body

    def clean(self):
        cleaned_data = super(SecretEntryForm, self).clean()
        password = cleaned_data.get('password', None)
        identifier = cleaned_data.get('identifier', None)
        pvt_key_pem = cleaned_data.get('body', None)
        # Only validate this field if we are dealing with a private key
        if identifier and is_private_key(identifier) and pvt_key_pem:
            (valid, key_pair) = cautils.pemPrivateKeyToPKey(
                str(pvt_key_pem),
                str(password)
            )
            # PASSWORD VALIDATION MUST OCCUR HERE
            if valid is not None:
                self.add_error('password', _("Invalid private key password"))
            else:
                (error, new_pvt_key) = cautils.pKeyToPEMPrivateKey(
                    key_pair,
                    str(settings.SECRET_KEY)
                )
                if error is not None:
                    cleaned_data['body'] = new_pvt_key

        return cleaned_data

    def save(self, commit=True):
        """ Overriding the ModelForm save method """
        # Only save if a file was submitted
        if not self.cleaned_data['body']:
            commit = False

        return super(SecretEntryForm, self).save(commit)


SecretEntryFormset = forms.modelformset_factory(
    SecretEntry,
    extra=0,
    form=SecretEntryForm,
    min_num=(
        len(
            SecretEntry.Identifier.choices
        )
    ),
    max_num=(
        len(
            SecretEntry.Identifier.choices
        )
    ),
)
