from __future__ import unicode_literals

from django.utils.translation import ugettext as _
from django.utils import timezone, formats
from django.apps import apps

from reportlab.platypus import Indenter

from evotedocs.documents import PdfDocument


class ElectionMinutePDF(PdfDocument):

    def set_story(self, **kwargs):
        election = kwargs.get('election')
        context = kwargs.get('context')
        now = formats.date_format(timezone.now(), 'DATETIME_FORMAT')

        self.story.append(self.title(
            _("Election minute of") + " " + election.identifier))
        self.add_field_description(
            _("Electoral process"), election.electoral_process.identifier)
        self.add_field_description(_("Election"), election.identifier)
        self.add_field_description(_("Date"), now)
        self.story.append(self.space(height=20))

        self.story.append(Indenter(-15, 0))
        self.story.append(self.hr())
        self.story.append(Indenter(0, 0))

        self.add_votes_count(election, context['total_votes_weight'],
                             context['total_votes'])

        self.story.append(Indenter(15, 0))
        self.story.append(self.space(height=5))
        self.add_table(_("Results"),
                       self.get_results_table(context['results_data']))
        self.add_table(_("Adicional info"),
                       self.get_results_extra_table(context['results_extra']))

    def add_votes_count(self, election, total_votes_weight, total_votes):
        """ Adds table with count of votes """
        if election.weight_votes:
            self.story.append(self.votes_table(
                [[_('Total votes with weight:'),
                  _('Total votes without weight:')],
                 [total_votes_weight, total_votes]]
            ))
        else:
            self.story.append(self.votes_table(
                [[_('Total votes in ballot box:')], [total_votes]]
            ))

    def add_field_description(self, label, value):
        """ Adds the block of pure-sec-title and field """
        self.story.append(self.pure_sec_title(label))
        self.story.append(self.space(height=5))
        self.story.append(Indenter(15, 0))
        self.story.append(self.body(value))
        self.story.append(Indenter(-15, 0))

    def add_table(self, label, table):
        """ Adds the block of pure-sec-title and table """
        self.story.append(self.pure_sec_title(label))
        self.story.append(self.space())
        self.story.append(Indenter(0, 15))
        self.story.append(table)
        self.story.append(Indenter(0, -15))

    def get_results_table(self, data):
        """ Returns the table of results """
        table = []
        table.append([
            self.label(_('Candidate')),
            self.label(_('Total votes')),
            self.label(_('Percentage')),
            self.label(_('Total seats'))
        ])
        for elem in data:
            cid = _('Blank votes')
            if elem['cid'] != 'blank':
                Candidate = apps.get_model("evoteelections.Candidate")  # noqa
                cid = Candidate.objects.get(pk=elem['cid'])

            table.append([
                self.body(str(cid)),
                self.body(str(elem['total'])),
                self.body(str(elem['perc']) + "%"),
                self.body(str(elem['seats']))
            ])
        return self.long_table(table, header=True)

    def get_results_extra_table(self, data):
        """ Returns the table of extra info """
        table = []
        table.append([
            self.label(_('Total invalid votes')),
            self.label(_('Total votes submitted')),
            self.label(_('Total votes submitting')),
            self.label(_('Total votes unsubmitted'))
        ])
        for elem in data:
            table.append([
                self.body(str(elem['invalid'])),
                self.body(str(elem['submitted'])),
                self.body(str(elem['submitting'])),
                self.body(str(elem['unsubmitted']))
            ])
        return self.long_table(table, header=True)
