from __future__ import unicode_literals

from django.utils.translation import ugettext as _
from django.db.models import F

from evotedocs.documents import PdfDocument

from reportlab.platypus import Indenter


class ElectoralProcessPdf(PdfDocument):

    def set_story(self, **kwargs):
        electoral_process = kwargs.get('electoral_process')

        if electoral_process.is_draft():
            self.story.append(self.title(_("Electoral Process creation")))
        else:
            self.story.append(self.title(_("Electoral Process") + " " +
                                         electoral_process.identifier))

        self.story.append(self.pure_sec_title(_("Summary")))
        self.story.append(self.space())
        self.story.append(self.pure_sec_title("1. " + _("General Data")))
        self.story.append(self.get_elecproc_table(electoral_process))

        # Adds all elections
        for i, election in enumerate(electoral_process.elections.all()):
            self.story.append(self.pure_sec_title(
                "1." + str(i + 1) + " " + _('General Data | Election ') +
                election.identifier))
            self.story.append(self.get_election_table(election))
            self.story.append(self.hr(space_after=10))
            self.story.append(self.label(_('Electoral calendar')))
            self.story.append(Indenter(15, 15))
            self.get_electoral_calendar(election)
            self.story.append(Indenter(-15, -15))

        self.story.append(self.pure_sec_title(
            "3. " + _("Electoral Comission")))
        self.story.append(self.space())
        self.story.append(Indenter(-15, 0))
        self.story.append(self.get_members_table(electoral_process))

    def get_elecproc_table(self, electoral_process):
        """ Returns the table with electoral process identification """
        table_data = ([[
            '',
            self.label("Electoral process identifier"),
            self.label("Description")
        ], [self.image(electoral_process.logo),
            self.body(electoral_process.identifier),
            self.body(electoral_process.description)]])

        return self.table(table_data, col_widths=[100, 200, 200] * 2)

    def get_election_table(self, election):
        """ Returns the table with election identification """
        data = ([[
            self.label(_("Election identifier")),
            self.label(_("Description"))
        ], [
            self.body(election.identifier),
            self.body(election.description)
        ]])

        return self.table(data)

    def get_electoral_calendar(self, election):
        """ Appends to story the electoral calendar of election """
        self.story.append(self.pure_sec_title(_("Proposition dates")))
        self.story.append(self.get_date_table(
            _("Begin date"), _("End date"),
            self.get_formated_date(election.proposition_begin_date),
            self.get_formated_date(election.proposition_end_date)))
        self.story.append(self.pure_sec_title(_("Electoral rolls")))
        self.story.append(self.get_date_table(
            _("Publication date before verification"),
            _("Publication date after verification and correction"),
            self.get_formated_date(election.roll_begin_date),
            self.get_formated_date(election.roll_end_date)))
        self.story.append(self.pure_sec_title(_("Voting")))
        self.story.append(self.get_date_table(
            _("Begin date"), _("End date"),
            self.get_formated_date(election.voting_begin_date),
            self.get_formated_date(election.voting_end_date)))
        self.story.append(self.pure_sec_title(_("Electoral act")))
        self.story.append(self.get_date_table(
            _("Date of publication"),
            '',
            self.get_formated_date(election.minute_publication_date),
            ''))
        self.story.append(self.get_date_table(
            _("Verification begin date"),
            _("Verification end date"),
            self.get_formated_date(election.minute_begin_date),
            self.get_formated_date(election.minute_end_date))
        )

    def get_date_table(self, id_one, id_two, field_one, field_two):
        """ Returns the table with electoral process identification """
        desc_one = self.body(_('GMT Timezone')) if field_one else ''
        desc_two = self.body(_('GMT Timezone')) if field_two else ''

        data = ([[self.label(id_one), '', self.label(id_two), ''],
                [field_one, desc_one, field_two, desc_two]])

        return self.table(data)

    def get_members_table(self, electoral_process):
        """ Returns the table of members """
        table_members = []
        table_members.append([
            self.label('#'),
            self.label(_('Name')),
            self.label(_('Role')),
            self.label(_('Designation')),
            self.label(_('Email address')),
            self.label(_('Contact Phone Number'))
        ])
        for i, member in enumerate(electoral_process.electoral_members.all()):
            table_members.append([
                self.body(str(i + 1)),
                self.body(member.user.get_full_name()),
                self.body(member.get_role_display()),
                self.body(member.designation),
                self.body(member.user.get_email()),
                self.body(str(member.user.get_phone()))
            ])

        return self.long_table(
            table_members, header=True,
            col_widths=[20, 100, 100, 100, 100, 100])


class ElectoralRollPdf(PdfDocument):

    def set_story(self, **kwargs):
        electoral_roll = kwargs.get('electoral_roll')
        self.story.append(self.title(
            _("Electoral Roll of ") +
            electoral_roll.election.identifier))
        self.add_header_table(electoral_roll)
        self.add_voters_table(electoral_roll)

    def add_header_table(self, electoral_roll):
        """ Adds the header of page to story """
        table_data = []
        table_data.append([
            self.get_reload_icon(),
            _('Version:') + " " + electoral_roll.version,
            self.get_calendar_icon(),
            _('Publication date:') + " " + self.get_formated_date(
                electoral_roll.published_on, body=False, icon=False)
        ])
        self.story.append(self.custom_header_table(
            table_data, col_widths=[15, 150, 15, 150]))

    def add_voters_table(self, electoral_roll):
        """ Adds table of voters to story """
        table_data = []
        table_data.append([
            _('Name'),
            _('Personnel Number'),
            _('Category')
        ])
        for voter in electoral_roll.election_voters.annotate(
                category_name=F("category__designation"))[:1000]:
            table_data.append([
                voter.name,
                voter.personnel_number,
                voter.category_name
            ])
        self.story.append(self.long_table(table_data, header=True))
