from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import get_object_or_404
from django.contrib import messages
from django.conf import settings
from django.http import Http404


from evoteajaxforms.views import AjaxResponseAction
from evotevotings.processes.models import (
    ElectoralProcess, ElectoralMember, ElectoralAction)

from .registers import action_validation
from .menus import Menu


class ActionValidationMixin(object):
    """ """
    action_validation_pk = None
    electoral_action = None

    def get_action_validation_pk(self):
        return self.action_validation_pk

    def get_action_validation_key(self):
        return self.get_object().pk

    def is_action_valid(self):
        action = self.get_electoral_action()
        if action and action.strength == \
                ElectoralAction.Strength.SIMPLE:
            return True
        return action_validation.validate(
            self.request, self.get_action_validation_key(),
            self.get_action_validation_pk())

    def get_action_validation_url(self):
        action = self.get_electoral_action()
        if action and action.strength == ElectoralAction.Strength.SIMPLE:
            return ""
        return reverse_lazy("management:action-validate", kwargs={
            'process_pk': self.get_action_validation_key(),
            'pk': self.get_action_validation_pk()
        })

    def get_context_data(self, **kwargs):
        context = super(ActionValidationMixin, self).get_context_data(**kwargs)
        context.update({
            'action_validation_url': self.get_action_validation_url()
        })
        return context

    def get_electoral_action(self):
        if self.get_action_validation_pk() == 'manage':
            return None
        elif not self.electoral_action:
            self.electoral_action = ElectoralAction.objects.filter(
                action_id=self.get_action_validation_pk(),
                electoral_process_id=self.get_action_validation_key()).first()
        return self.electoral_action

    def dispatch(self, request, *args, **kwargs):
        """ """
        if request.method == 'POST' and not self.is_action_valid():
            messages.error(
                request, _("You are trying to made an invalidated operation."))
            return self.json_to_response(action=AjaxResponseAction.REFRESH)
        return super(ActionValidationMixin, self)\
            .dispatch(request, *args, **kwargs)


class MessageMixin(object):
    """ """
    selected_menu = Menu.ELECPROCESS

    def get_context_data(self, **kwargs):
        slug = self.kwargs.get('slug', None)
        context = super(MessageMixin, self).get_context_data(
            **kwargs)
        if slug:
            process = get_object_or_404(ElectoralProcess, slug=slug)
            context.update({
                'obj': process,
                'member': get_object_or_404(
                    ElectoralMember, user=self.request.user,
                    electoral_process=process)
            })
        return context


class MessageViewManageMixin(object):

    def get_form_kwargs(self):
        """ """
        kwargs = super(MessageViewManageMixin, self).get_form_kwargs()
        kwargs.update({
            'obj': ElectoralProcess.objects.filter(
                slug=self.kwargs.get('slug', '')).first()
        })
        return kwargs

    def get_initial(self):
        initial = super(MessageViewManageMixin, self).get_initial()
        user = self.request.user
        is_member = getattr(user, 'is_electoral_member', False)
        if is_member and not self.response:
            process = get_object_or_404(
                ElectoralProcess, slug=self.kwargs['slug'])
            initial.update({
                'name': _('Electoral comission to ') + process.identifier +
                " - " + user.name,
                'email': getattr(settings, "EVOTE_MAILING_NOREPLY_DEFAULT", '')
            })
        return initial


class ElectionDetailMixin(object):

    def dispatch(self, request, *args, **kwargs):
        if not hasattr(self, 'object'):
            self.object = self.get_object()
        if not self.object.can_change() or not self.can_change():
            raise Http404()
        return super(ElectionDetailMixin, self).dispatch(
            request, *args, **kwargs)
