from __future__ import unicode_literals

from django.db.models import Count
from django.db import models

from evoteusers.models import User
from evotecore.utils import compare_data


class ElectoralRollQuerySet(models.QuerySet):

    def get_published_of_process(self):
        """Get published electoral roll on the process"""
        return self.filter(published_on__isnull=False).first()

    def get_published(self, election_id):
        """Get published electoral roll on the election"""
        return self.filter(
            election_id=election_id, published_on__isnull=False).first()

    def get_to_publish(self, election_id):
        """Get electoral roll that can be published"""
        to_publish = self.filter(
            election_id=election_id, published_on__isnull=True, imported=True)
        published = self.get_published(election_id)
        if published:
            to_publish = to_publish.filter(created__gt=published.created)
        return to_publish.first()

    def is_importing(self, election_id):
        """ """
        return self.filter(election_id=election_id, imported=None)


class ElectionVoterQuerySet(models.QuerySet):
    """ """

    def votes_status_count(self):
        """ """
        status_count = dict((name, 0) for status, name in
                            self.model.VoteStatus.names.items())
        # order_by empty to avoid group by name (default ordering)
        counts = self.values('vote_status')\
            .annotate(total=Count('vote_status')).order_by()
        for count in counts:
            name = self.model.VoteStatus.names[count['vote_status']]
            status_count[name] = count['total']
        return status_count

    def assign_perms(self):
        """ """
        qs = self.select_related('user', 'election__electoral_process')\
            .order_by()
        for voter in qs:
            voter.assign_perm()

    def remove_perms(self):
        """ """
        qs = self.select_related('user', 'election__electoral_process')\
            .order_by()
        for voter in qs:
            voter.remove_perm()


class ElectionVoterBaseManager(models.Manager):
    """ """

    def bulk_import(self, electoral_roll, data):
        """Import voter for determined election. Create associated user if
        necessary to allow the voter login in the system.
        Associate permissions to allow user view the election and associated
        electoral process."""
        voters_user_ids = []
        voters_to_create = []
        registered_emails = User.objects.values_list("email", flat=True)
        for row in data:
            voter_user_id = None
            row["category_id"] = electoral_roll.election.categories.only("id")\
                .get(slug=row["category"]).pk
            if row["email"] not in registered_emails:
                voter_user_id = User.objects.create(
                    name=row["name"],
                    email=row["email"],
                    primary_phone=row["phone"]
                ).pk
            else:
                # user already exists in database
                try:
                    voter = self.get(
                        election_id=electoral_roll.election_id,
                        user__email=row["email"])
                    self.efficient_update(voter, row, [
                        "name",
                        "category_id",
                        "personnel_number"
                    ])
                    voters_user_ids.append(voter.user_id)
                except self.model.DoesNotExist:
                    voter_user_id = User.objects.only("id")\
                        .get(email=row["email"]).pk
            if voter_user_id:
                voters_user_ids.append(voter_user_id)
                voters_to_create.append(self.model(
                    name=row["name"],
                    user_id=voter_user_id,
                    election_id=electoral_roll.election_id,
                    category_id=row["category_id"],
                    personnel_number=row["personnel_number"],
                ))
        self.bulk_create(voters_to_create)
        election_voters = self.filter(election_id=electoral_roll.election_id,
                                      user_id__in=voters_user_ids)
        electoral_roll.election_voters.add(*election_voters)
        return election_voters

    def efficient_update(self, voter, data, keys):
        """Only update if data changed, otherwise do nothing"""
        need_update = not compare_data(voter, data, keys)
        if need_update:
            for key in keys:
                setattr(voter, key, data[key])
            voter.save(update_fields=keys)


ElectionVoterManager = ElectionVoterBaseManager.from_queryset(
    ElectionVoterQuerySet)
