from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import python_2_unicode_compatible
from django.contrib.contenttypes.models import ContentType
from django.utils import timezone
from django.conf import settings
from django.db import models

from evotevotings.elections.models import Category, Election
from django_extensions.db.models import TimeStampedModel
from evotelogs.decorators import auditable
from guardian.shortcuts import assign_perm, remove_perm
from evotedocs.models import Document
from autoslug import AutoSlugField

from .managers import ElectionVoterManager, ElectoralRollQuerySet


def populate_electoral_roll(instance):
    return instance.document.designation


@auditable
@python_2_unicode_compatible
class ElectoralRoll(TimeStampedModel):
    """ """
    election = models.ForeignKey(
        Election, verbose_name=_("election"), related_name="electoral_rolls",
        on_delete=models.CASCADE)
    document = models.ForeignKey(
        Document, verbose_name=_("document"), related_name="+",
        on_delete=models.PROTECT)
    pdf_document = models.ForeignKey(
        Document, verbose_name=_("PDF document"), related_name="+",
        null=True, default=None)
    slug = AutoSlugField(
        populate_from=populate_electoral_roll,
        unique=True, editable=False)
    published_on = models.DateTimeField(
        _("published on"), null=True, default=None)
    imported = models.NullBooleanField(default=None)
    objects = ElectoralRollQuerySet.as_manager()

    class Meta:
        ordering = ("-created",)
        verbose_name = _("electoral roll")
        verbose_name_plural = _("electoral rolls")

    def __str__(self):
        return self.document.designation

    @property
    def audit_group(self):
        elec_proc = self.election.electoral_process
        group_type = ContentType.objects.get_for_model(elec_proc)
        return (group_type.id, elec_proc.pk)

    @property
    def version(self):
        return self.created.strftime('%Y%m%d%H%M')

    @property
    def published(self):
        return bool(self.published_on)

    def publish(self):
        """Update date of publication, mark voters as active and inactive, and
        update all permissions"""
        # mark new voters as active
        self.election_voters.update(is_active=True)
        # assign permissions to active voters
        self.election_voters.assign_perms()
        # unmark voters wich are not listed as inactive
        voter_ids = list(self.election_voters.values_list('id', flat=True))
        inactivate_voters = ElectionVoter.objects.exclude(pk__in=voter_ids)
        if inactivate_voters.exists():
            inactivate_voters.update(is_active=False)
            # remove permissions of inactive voters
            inactivate_voters.remove_perms()
        # update published date
        self.published_on = timezone.now()
        self.save(update_fields=['published_on'])


class VoteValidationMixin(models.Model):
    """Mixin with vote validation fields
    """
    pin_value = models.CharField(
        _("PIN"),
        max_length=64,
        editable=False,
        null=True)
    token_send_through = models.CharField(
        _("Authenticate using PIN (Email/SMS) or .Gov"),
        max_length=32,
        editable=False,
        null=True)
    token_expiration_date = models.DateTimeField(
        _("Authentication token validation date"),
        editable=False,
        null=True)
    token_validated = models.BooleanField(
        _("Valid token"),
        default=False)

    class Meta:
        abstract = True

    def is_token_valid(self, pin_value=None):
        """ """
        now = timezone.now()
        saml_confs = getattr(settings, 'SAML', '')
        sess_key = saml_confs.get('session_dict', None)
        if self.token_send_through != sess_key:
            if pin_value != self.pin_value:
                return False

        if now > self.token_expiration_date:
            return False

        return True

    def set_token_validated(self, value):
        """ """
        self.token_validated = value
        self.save(update_fields=['token_validated'])


@auditable
@python_2_unicode_compatible
class ElectionVoter(TimeStampedModel, VoteValidationMixin):
    """ """
    class VoteStatus:
        NO = 0
        YES = 1
        PENDING = 2

        choices = (
            (NO, _("Unsubmitted")),
            (YES, _("Submitted")),
            (PENDING, _("Submitting")),
        )
        names = {
            NO: 'unsubmitted',
            YES: 'submitted',
            PENDING: 'submitting',
        }

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, verbose_name=_("user"),
        on_delete=models.CASCADE, related_name="election_voters")
    election = models.ForeignKey(
        Election, verbose_name=_("election"), related_name="election_voters",
        on_delete=models.CASCADE)
    category = models.ForeignKey(
        Category, verbose_name=_("category"), related_name="election_voters",
        on_delete=models.PROTECT)
    electoral_rolls = models.ManyToManyField(
        ElectoralRoll, verbose_name=_("electoral rolls"),
        related_name="election_voters")
    name = models.CharField(_("name"), max_length=255, db_index=True)
    personnel_number = models.CharField(_("personnel number"), max_length=64,
                                        db_index=True)
    is_active = models.BooleanField(_("is active"), default=False)
    blind_vote_hash = models.CharField(
        _("blind user cyphered vote hash"), max_length=255, editable=False,
        db_index=True, null=True)
    vote_status = models.PositiveSmallIntegerField(
        _("vote status"), choices=VoteStatus.choices, default=VoteStatus.NO)
    objects = ElectionVoterManager()

    class Meta:
        ordering = ('name',)
        verbose_name = _("voter")
        verbose_name_plural = _("voters")
        index_together = ['election', 'vote_status']

    def __str__(self):
        return "{} ({})".format(self.name, self.personnel_number)

    @property
    def audit_group(self):
        elec_proc = self.election.electoral_process
        group_type = ContentType.objects.get_for_model(elec_proc)
        return (group_type.id, elec_proc.pk)

    @property
    def is_vote_available(self):
        """Checks if this ElectionVoter vote is still pending
           Can either be in PENDING after 2 minutes or
           in the NO status"""
        if(self.vote_status == self.VoteStatus.NO):
            return (0, True)
        if(self.vote_status == self.VoteStatus.YES):
            return (1, False)

        current_date = timezone.now()
        threshold = current_date - timezone.timedelta(minutes=2)
        available = self.modified < threshold

        if(available):
            return (0, True)
        else:
            return (2, False)

    def assign_perm(self):
        """ """
        assign_perm("view_election", self.user, self.election)
        assign_perm("vote_election", self.user, self.election)
        assign_perm("view_electoralprocess", self.user,
                    self.election.electoral_process)

    def remove_perm(self):
        """ """
        remove_perm("view_election", self.user, self.election)
        remove_perm("vote_election", self.user, self.election)
        remove_perm("view_electoralprocess", self.user,
                    self.election.electoral_process)
